__title__ = 'Django Platform Data Service'
__version__ = '0.0.1'
__author__ = 'Komol Nath Roy'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020 Komol Nath Roy'

VERSION = __version__

default_app_config = 'django_pds.apps.DjangoPdsConfig'
